import 'dart:developer';

import 'package:filesize/filesize.dart';
import 'package:flutter/material.dart';
import 'package:jiffy/jiffy.dart';

import '../../../../../widget/centeredLeading.dart';
import '../../../../../widget/list_view_util.dart';
import '../../../infrastructure/loadableState/loadable_state.dart';
import '../../../infrastructure/loadableState/view/loadable_state_consumer.dart';
import '../../../infrastructure/utilityWidgets/bloc_module.dart';
import '../bloc/files_bloc.dart';
import '../bloc/files_event.dart';
import '../bloc/files_state.dart';

class FilesView extends StatelessWidget {
  const FilesView({super.key});

  @override
  Widget build(BuildContext context) => BlocModule<FilesBloc, LoadableState<FilesState>>(
    create: (context) => FilesBloc(),
    autoRebuild: true,
    child: (context, bloc, state) {
      var goBackButton = !bloc.canGoBack() ? null : IconButton(
        icon: const Icon(Icons.arrow_back),
        onPressed: () {
          bloc.add(EnterFolder(bloc.goBackLocation()));
        },
      );
      return PopScope(
        canPop: false,
        onPopInvoked: (didPop) => bloc.add(EnterFolder(bloc.goBackLocation())),
        child: Scaffold(
            appBar: AppBar(
              leading: goBackButton,
              title: Text(bloc.getCurrentFolderName()),


              actions: [
                IconButton(onPressed: () {
                  log(bloc.innerState?.toJson().toString() ?? 'leer');
                }, icon: const Icon(Icons.bug_report)),
                IconButton(onPressed: () {
                  bloc.add(EnterFolder('/'));
                }, icon: const Icon(Icons.home)),
              ],
            ),
            body: LoadableStateConsumer<FilesBloc, FilesState>(
                child: (state, loading) => ListViewUtil.fromList<File>(bloc.getVisibleFiles(), (file) => ListTile(
                  leading: CenteredLeading(Icon(file.isFolder ? Icons.folder : Icons.description_outlined)),
                  title: Text(file.name),
                  subtitle: file.isFolder
                      ? Text('geändert ${Jiffy.parseFromDateTime(file.updatedAt ?? DateTime.now()).fromNow()}')
                      : Text('${filesize(file.size)}, ${Jiffy.parseFromDateTime(file.updatedAt ?? DateTime.now()).fromNow()}'),
                  trailing: Icon(file.isFolder ? Icons.arrow_right : null),
                  onTap: () {
                    log(file.path);
                    if(!file.isFolder) return;
                    bloc.add(EnterFolder(file.path));
                  },
                ))
            )
        ),
      );
    },
  );
}
